@@ -255,6 +255,7 @@ SS2DLM<-function(SSdir,nsim=48,proyears=50,length_timestep=NA,Name=NULL,Source="
   
   OM@cpars<-list(V=V,Perr=Perr,Wt_age=Wt_age2,K=K,Linf=Linf,hs=hs,Find=Find)
   # attr(OM, "build") <- "SS2DLM"
+  OM@seed <- 1
   OM
   
 }
@@ -283,16 +284,20 @@ LinInterp<-function(x,y,xlev,ascending=F,zeroint=F){
   }
   
   close<-which.min((x[cond]-xlev)^2)
+  
   ind<-c(close,close+(x[close]<xlev)*2-1)
   ind <- ind[ind <= length(x)]
   if (length(ind)==1) ind <- c(ind, ind-1)
+  if (min(ind)==0) ind <- 1:2
   ind<-ind[order(ind)]
+
   pos<-(xlev-x[ind[1]])/(x[ind[2]]-x[ind[1]])
-  y[ind[1]]+pos*(y[ind[2]]-y[ind[1]])
+  max(1,y[ind[1]]+pos*(y[ind[2]]-y[ind[1]]))
   
 }
 
 
+
 #' A function that samples multivariate normal (logspace) variables 
 #'
 #' @param xmu The mean (normal space) of the first (x) variable 
